#pragma once

#include "FloatSize.h"
#include "Image.h"
#include <wtf/Ref.h>
#include <wtf/RefPtr.h>

#if USE( TSRENDER )
namespace WebCore
{
class CanvasImage final : public Image
{
public:
	static RefPtr<CanvasImage> create( const IntSize &size, bool isDeferred )
	{
		return adoptRef( new CanvasImage( size, isDeferred ) );
	}

	virtual ~CanvasImage();

	virtual bool currentFrameKnownToBeOpaque() const override;
	virtual FloatSize size( ImageOrientation = ImageOrientation::FromImage ) const override;
	virtual void destroyDecodedData( bool destroyAll = true ) override;
	virtual ImageDrawResult draw( GraphicsContext &context, const FloatRect &dstRect, const FloatRect &srcRect, const ImagePaintingOptions & = {} ) override;

	bool hasSingleSecurityOrigin() const override;
	void setContainerSize( const FloatSize &size ) override;
	bool usesContainerSize() const override;
	bool hasRelativeWidth() const override;
	bool hasRelativeHeight() const override;
	void computeIntrinsicDimensions( Length &intrinsicWidth, Length &intrinsicHeight, FloatSize &intrinsicRatio ) override;

	virtual void drawPattern( GraphicsContext &, const FloatRect &destRect, const FloatRect &srcRect, const AffineTransform &patternTransform, const FloatPoint &phase, const FloatSize &spacing, const ImagePaintingOptions & = {} ) override;

protected:
	CanvasImage( const IntSize &size, bool isDeferred );

private:
	friend class ImageBuffer;
};

} // namespace WebCore
#endif // #if USE( TSRENDER )