/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <config.h>
#include "Gradient.h"
#include "TSRender.h"
#include "GraphicsContext.h"
#include "PlatformContextTSRender.h"
#include "NotImplemented.h"

#if USE( TSRENDER )
namespace WebCore
{
void Gradient::platformDestroy()
{
	m_gradient = nullptr;
}


PlatformGradient Gradient::createPlatformGradient( float globalAlpha )
{
	m_gradient = WTF::switchOn(
		m_data,
		[&]( const LinearData &data ) -> PlatformGradient
		{
			return tsRender::Gradient::CreateLinear( tsRender::Point( data.point0.x(), data.point0.y() ), tsRender::Point( data.point1.x(), data.point1.y() ) );
		},
		[&]( const RadialData &data ) -> PlatformGradient
		{
			tsRender::Point center( ( data.point0.x() + data.point1.x() ) / 2, ( data.point0.y() + data.point1.y() ) / 2 );
			return tsRender::Gradient::CreateRadial( center, data.startRadius, data.endRadius );
		},
		[&]( const ConicData & ) -> PlatformGradient
		{
			notImplemented();
			assert( false );
			return nullptr;
		} );

	if ( !m_gradient )
	{
		return nullptr;
	}

	for ( int i = 0; i < m_stops.size(); ++i )
	{
		auto &stop = m_stops[i];
		m_gradient->AddColorStop( stop.offset,
			tsRender::ColorFromRGBA( stop.color.red(), stop.color.green(), stop.color.blue(), stop.color.alpha() * globalAlpha ) );
	}

	m_gradient->Transform( ConvertAffineTransformToTSRender( m_gradientSpaceTransformation ) );
	return m_gradient;
}


void Gradient::fill( GraphicsContext &context, const FloatRect &rect )
{
	auto gradient = createPlatformGradient( context.alpha() );
	if ( !gradient )
	{
		return;
	}
	auto path = tsRender::Path::Create();
	path->Rect( ConvertFloatRectToTSRender( rect ) );


	ASSERT( context.hasPlatformContext() );
	context.platformContext()->getSceneBuilder()->FillPath( *path, *gradient );
}
} // namespace WebCore
#endif // #if USE( TSRENDER )