/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#if USE( TSRENDER )
#include "GraphicsContext.h"
#include "TSRender.h"

namespace WebCore
{
class PlatformContextTSRender
{
	WTF_MAKE_FAST_ALLOCATED;
	WTF_MAKE_NONCOPYABLE( PlatformContextTSRender );

public:
	class TSRenderState
	{
	public:
		TSRenderState() {}
		TSRenderState( const TSRenderState &state )
			: m_imageInterpolationQuality( state.m_imageInterpolationQuality )
			, m_lineCap( state.m_lineCap )
			, m_lineJoin( state.m_lineJoin )
			, m_lineDash( state.m_lineDash )
			, m_lineDashOffset( state.m_lineDashOffset )
			, m_miterLimit( state.m_miterLimit )
			, m_alpha( state.m_alpha )
		{
		}

		InterpolationQuality m_imageInterpolationQuality = InterpolationQuality::Default;
		LineCap m_lineCap = LineCap::ButtCap;
		LineJoin m_lineJoin = LineJoin::MiterJoin;
		DashArray m_lineDash;
		float m_lineDashOffset;
		float m_miterLimit = 10.0f;
		float m_alpha = 1.0f;
	};

	PlatformContextTSRender( tsRender::RefPtr<tsRender::SceneBuilder> );
	~PlatformContextTSRender();

	GraphicsContextPlatformPrivate *graphicsContextPrivate() { return m_graphicsContextPrivate; }
	void setGraphicsContextPrivate( GraphicsContextPlatformPrivate *graphicsContextPrivate ) { m_graphicsContextPrivate = graphicsContextPrivate; }

	void save();
	void restore();

	void setLineCap( LineCap lineCap );
	LineCap lineCap() const;

	void setLineJoin( LineJoin lineJoin );
	LineJoin lineJoin() const;

	void setLineDash( const DashArray &dashes, float dashOffset );
	const float *lineDashData() const;
	size_t lineDashSize() const;
	float lineDashOffset() const;

	void setMiterLimit( float miter );
	float miterLimit() const;

	tsRender::RefPtr<tsRender::SceneBuilder> getSceneBuilder() const;

private:
	GraphicsContextPlatformPrivate *m_graphicsContextPrivate = nullptr;
	TSRenderState m_renderState;
	tsRender::RefPtr<tsRender::SceneBuilder> m_sceneBuilder = nullptr;
};

} // namespace WebCore

#endif // #if USE( TSRENDER )
