/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <config.h>
#include "Color.h"
#include "IntRect.h"
#include "FloatRect.h"
#include "FloatRoundedRect.h"
#include "GraphicsTypes.h"
#include "tsRender/private/Render.h"
#include "tsRender/private/SceneBuilder.h"

#if USE( TSRENDER )
namespace WebCore
{
class GraphicsContext;
class TSRenderGlobal
{
public:
	static TSRenderGlobal &getInstance()
	{
		static TSRenderGlobal instance;
		return instance;
	}

	void initRender( tsRender::RefPtr<tsRender::Render> pRender )
	{
		m_pRender = pRender;
	}

	void shutdownRender()
	{
		m_pRender = nullptr;
	}

	const tsRender::RefPtr<tsRender::Render> getRender() const { return m_pRender; }

private:
	TSRenderGlobal() {}
	WTF_MAKE_NONCOPYABLE( TSRenderGlobal )

	tsRender::RefPtr<tsRender::Render> m_pRender = nullptr;
};

tsRender::Color ConvertColorToTSRender( Color cl );
tsRender::FloatRect ConvertFloatRectToTSRender( const FloatRect &rect );
tsRender::RoundedRect ConvertRoundedRectToTSRender( const FloatRoundedRect &roundedRect );
tsRender::Point ConvertPointToTSRender( const FloatPoint &pt );
AffineTransform ConvertFromWebCoreMatrixToTSRenderMatrix( tsRender::AffineTransform &mat );
tsRender::AffineTransform ConvertAffineTransformToTSRender( const AffineTransform &mat );
tsRender::LineCap ConvertLineCapToTSRender( LineCap e );
tsRender::LineJoin ConvertLineJoinToTSRender( LineJoin e );
void adjustFocusRingColor( Color &color );
void adjustFocusRingLineWidth( float &width );
} // namespace WebCore

#endif // #if USE( TSRENDER )
