/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <stdint.h>

namespace Telescope
{

typedef int64_t TEL_NET_SOCKET;
typedef uint16_t TEL_NET_ADDRESS_FAMILY;
typedef uint16_t TEL_NET_IN_PORT;
typedef uint32_t TEL_NET_IN_ADDR;
typedef uint32_t TEL_NET_SOCK_TYPE;
typedef uint32_t TEL_NET_IPPROTO_TYPE;

constexpr TEL_NET_ADDRESS_FAMILY TEL_NET_AF_INET = 2;
constexpr TEL_NET_SOCK_TYPE TEL_NET_SOCK_STREAM = 1;
constexpr TEL_NET_IPPROTO_TYPE TEL_NET_IPPROTO_TCP = 6;
constexpr TEL_NET_SOCKET TEL_NET_INVALID_SOCKET = -1;
constexpr uint32_t TEL_NET_INADDR_ANY = 0x00000000;
constexpr int TEL_NET_SOL_SOCKET = 0xffff;

constexpr int TEL_NET_SO_REUSEADDR = 0x0004;
constexpr int TEL_NET_SO_REUSEPORT = 0x0200;
constexpr int TEL_NET_SO_SNDBUF = 0x1001;
constexpr int TEL_NET_SO_RCVBUF = 0x1002;
constexpr int TEL_NET_SO_NBIO = 0x1200;

constexpr uint32_t TEL_NET_POLLRDNORM = 0x0100;
constexpr uint32_t TEL_NET_POLLRDBAND = 0x0200;
constexpr uint32_t TEL_NET_POLLIN = ( TEL_NET_POLLRDNORM | TEL_NET_POLLRDBAND );
constexpr uint32_t TEL_NET_POLLWRNORM = 0x0010;
constexpr uint32_t TEL_NET_POLLOUT = TEL_NET_POLLWRNORM;

struct SockAddr_In
{
	TEL_NET_ADDRESS_FAMILY sin_family;
	TEL_NET_IN_PORT sin_port;
	TEL_NET_IN_ADDR sin_addr;
	uint8_t sin_zero[8];
};

struct SockAddr
{
	TEL_NET_ADDRESS_FAMILY sa_family;
	char sa_data[14];
};

static_assert( sizeof( SockAddr_In ) == sizeof( SockAddr ) );

struct PollEvent
{
	TEL_NET_SOCKET fd;
	uint64_t events;
	uint64_t revents;
};

} // namespace Telescope