/*
 * Copyright (C) 2005-2007, 2009, 2014-2015 Apple Inc. All rights reserved.
 * Copyright (C) 2010 Torch Mobile (Beijing) Co. Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if TELESCOPE_USING( TELESCOPE_TESTRUNNER )

#include "config.h"
#include "FrameLoadDelegate.h"

#include <JavaScriptCore/APICast.h>
#include <JavaScriptCore/JavaScriptCore.h>
#include "Tool/TestRunner/GCController.h"
#include "Tool/TestRunner/TestRunner.h"
#include "WebCoreTestSupport.h"
#include "WebFrame.h"
#include "WebView.h"
#include <WebCore/Frame.h>
#include <wtf/Assertions.h>
#include <wtf/NeverDestroyed.h>
#include <wtf/Vector.h>

namespace Telescope
{

static WTF::String descriptionSuitableForTestResult(WebFrame* webFrame)
{
    WebView* webView = webFrame->webView();
    if (!webView)
        return "";
        
    WebFrame* topLevelFrame = WebFrame::fromCoreFrame(*webView->TopLevelFrame());
    if (!topLevelFrame)
        return "";

    WTF::String frameName;
    webFrame->name(&frameName);
    bool isTopWebFrame = (webFrame == topLevelFrame);
    if (!frameName.length())
        return isTopWebFrame ? "main frame" : "";
    WTF::String frameDesc = isTopWebFrame ? "main frame" : "frame";
    frameDesc = frameDesc + " \"" + frameName + "\"";

    return frameDesc;
}

FrameLoadDelegate::FrameLoadDelegate()
    : m_gcController(makeUnique<GCController>())
    /*
    , m_accessibilityController(makeUnique<AccessibilityController>())
    , m_textInputController(makeUnique<TextInputController>())
    */
{
}

FrameLoadDelegate::~FrameLoadDelegate()
{
}

/*
HRESULT FrameLoadDelegate::QueryInterface(_In_ REFIID riid, _COM_Outptr_ void** ppvObject)
{
    if (!ppvObject)
        return E_POINTER;
    *ppvObject = nullptr;
    if (IsEqualGUID(riid, IID_IUnknown))
        *ppvObject = static_cast<IWebFrameLoadDelegate*>(this);
    else if (IsEqualGUID(riid, IID_IWebFrameLoadDelegate))
        *ppvObject = static_cast<IWebFrameLoadDelegate*>(this);
    else if (IsEqualGUID(riid, IID_IWebFrameLoadDelegatePrivate))
        *ppvObject = static_cast<IWebFrameLoadDelegatePrivate*>(this);
    else if (IsEqualGUID(riid, IID_IWebFrameLoadDelegatePrivate2))
        *ppvObject = static_cast<IWebFrameLoadDelegatePrivate2*>(this);
    else if (IsEqualGUID(riid, IID_IWebNotificationObserver))
        *ppvObject = static_cast<IWebNotificationObserver*>(this);
    else
        return E_NOINTERFACE;

    AddRef();
    return S_OK;
}

ULONG FrameLoadDelegate::AddRef()
{
    return ++m_refCount;
}

ULONG FrameLoadDelegate::Release()
{
    ULONG newRef = --m_refCount;
    if (!newRef)
        delete(this);

    return newRef;
}
*/

bool FrameLoadDelegate::didStartProvisionalLoadForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didStartProvisionalLoadForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());
        
    // Make sure we only set this once per test.  If it gets cleared, and then set again, we might
    // end up doing two dumps for one test.
    if (!topLoadingFrame && !done)
        topLoadingFrame = webFrame;

    return true; 
}

bool FrameLoadDelegate::didReceiveServerRedirectForProvisionalLoadForFrame(WebView* webView, WebFrame* webFrame)
{ 
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didReceiveServerRedirectForProvisionalLoadForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());

    return true;
}

bool FrameLoadDelegate::didChangeLocationWithinPageForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didChangeLocationWithinPageForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());
    return true;
}

bool FrameLoadDelegate::didFailProvisionalLoad(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didFailProvisionalLoad\n", descriptionSuitableForTestResult(webFrame).utf8().data());
    locationChangeDone(webFrame);
    return true;
}

bool FrameLoadDelegate::didCommitLoadForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didCommitLoadForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());
    return true;
}

bool FrameLoadDelegate::didReceiveTitle(WebView* webView, const WTF::String& title, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didReceiveTitle: %s\n", descriptionSuitableForTestResult(webFrame).utf8().data(), title.utf8().data());
        
    if (gTestRunner->dumpTitleChanges() && !done)
        TESTRUNNER_OUTPUT("TITLE CHANGED: '%s'\n", title.utf8().data());
        
    return true;
}

/*
HRESULT FrameLoadDelegate::didChangeIcons(_In_opt_ IWebView*, _In_opt_ IWebFrame* frame)
{
    // This feature is no longer supported. The stub is here to keep backwards compatibility
    // with the COM interface.
    return S_OK;
}
*/

void FrameLoadDelegate::processWork()
{
    // if another load started, then wait for it to complete.
    if (topLoadingFrame)
        return;

    // if we finish all the commands, we're ready to dump state
    if (!gTestRunner->waitToDump())
        gTestRunner->doDump();
}

void FrameLoadDelegate::resetToConsistentState()
{
    /*
    m_accessibilityController->resetToConsistentState();
    */
}

/*
typedef Vector<COMPtr<FrameLoadDelegate> > DelegateVector;
static DelegateVector& delegatesWithDelayedWork()
{
    static NeverDestroyed<DelegateVector> delegates;
    return delegates;
}

static UINT_PTR processWorkTimerID;

static void CALLBACK processWorkTimer(HWND hwnd, UINT, UINT_PTR id, DWORD)
{
    ASSERT_ARG(id, id == processWorkTimerID);
    ::KillTimer(hwnd, id);
    processWorkTimerID = 0;

    DelegateVector delegates;
    delegates.swap(delegatesWithDelayedWork());

    for (size_t i = 0; i < delegates.size(); ++i)
        delegates[i]->processWork();
}
*/

void FrameLoadDelegate::locationChangeDone(WebFrame* frame)
{
    if (frame != topLoadingFrame)
        return;
        
    topLoadingFrame = nullptr;

    if (gTestRunner->waitToDump())
        return;

    gTestRunner->doDump();
}

bool FrameLoadDelegate::didFinishLoadForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didFinishLoadForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());
        
    locationChangeDone(webFrame);
    return true;
}

bool FrameLoadDelegate::didFailLoad(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didFailLoadWithError\n", descriptionSuitableForTestResult(webFrame).utf8().data());

    locationChangeDone(webFrame);
    return true;
}

bool FrameLoadDelegate::willPerformClientRedirectToURL(WebView*, const WTF::String& url, double /*delaySeconds*/, double /*fireDate*/, WebFrame* frame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
    {
        TESTRUNNER_OUTPUT("%s - willPerformClientRedirectToURL: %s \n", descriptionSuitableForTestResult(frame).utf8().data(), url.utf8().data());
        //TODO: impl urlSuitableForTestResult to format the URL;
    }

    return true;
}

bool FrameLoadDelegate::didCancelClientRedirectForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didCancelClientRedirectForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());

    return true;
}

/*
HRESULT FrameLoadDelegate::willCloseFrame(_In_opt_ IWebView*, _In_opt_ IWebFrame*)
{
    return E_NOTIMPL;
}
*/

bool FrameLoadDelegate::windowScriptObjectAvailable(WebView*, JSContextRef, JSObjectRef)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("?? - windowScriptObjectAvailable\n");
        
    ASSERT_NOT_REACHED();

    return true;
}

/*
HRESULT FrameLoadDelegate::didClearWindowObject(IWebView*, JSContextRef, JSObjectRef, IWebFrame*)
{
    return E_NOTIMPL;
}
*/

bool FrameLoadDelegate::didClearWindowObjectForFrameInScriptWorld(WebView* webView, WebFrame* frame)
{
    ASSERT_ARG(webView, webView);
    ASSERT_ARG(frame, frame);
    if (!webView || !frame)
        return false;

    didClearWindowObjectForFrameInStandardWorld(frame);
    return true;
}

/*
void FrameLoadDelegate::didClearWindowObjectForFrameInIsolatedWorld(IWebFrame* frame, IWebScriptWorld* world)
{
    COMPtr<IWebFramePrivate> framePrivate(Query, frame);
    if (!framePrivate)
        return;

    JSGlobalContextRef ctx = framePrivate->globalContextForScriptWorld(world);
    if (!ctx)
        return;

    JSObjectRef globalObject = JSContextGetGlobalObject(ctx);
    if (!globalObject)
        return;

    JSObjectSetProperty(ctx, globalObject, JSRetainPtr<JSStringRef>(Adopt, JSStringCreateWithUTF8CString("__worldID")).get(), JSValueMakeNumber(ctx, worldIDForWorld(world)), kJSPropertyAttributeReadOnly, 0);
    return;
}
*/

void FrameLoadDelegate::didClearWindowObjectForFrameInStandardWorld(WebFrame* webFrame)
{
    JSGlobalContextRef context = webFrame->globalContext();
    JSObjectRef windowObject = JSContextGetGlobalObject(context);

    JSValueRef exception = 0;

    gTestRunner->makeWindowObject(context, windowObject, &exception);
    ASSERT(!exception);

    m_gcController->makeWindowObject(context, windowObject, &exception);
    ASSERT(!exception);

    /*
    m_accessibilityController->makeWindowObject(context, windowObject, &exception);
    ASSERT(!exception);

    m_textInputController->makeWindowObject(context, windowObject, &exception);
    ASSERT(!exception);

    JSStringRef eventSenderStr = JSStringCreateWithUTF8CString("eventSender");
    JSValueRef eventSender = makeEventSender(context, !parentFrame);
    JSObjectSetProperty(context, windowObject, eventSenderStr, eventSender, kJSPropertyAttributeReadOnly | kJSPropertyAttributeDontDelete, 0);
    JSStringRelease(eventSenderStr);
    */
    WebCoreTestSupport::injectInternalsObject(context);
}

bool FrameLoadDelegate::didFinishDocumentLoadForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didFinishDocumentLoadForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());

    if (!done) {
        unsigned pendingFrameUnloadEvents = webFrame->pendingFrameUnloadEventCount();
        if (pendingFrameUnloadEvents)
        {
            TESTRUNNER_OUTPUT("%s - has %u onunload handler(s)\n", descriptionSuitableForTestResult(webFrame).utf8().data(), pendingFrameUnloadEvents);
        }
    }

    return true;
}

bool FrameLoadDelegate::didHandleOnloadEventsForFrame(WebView* webView, WebFrame* webFrame)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("%s - didHandleOnloadEventsForFrame\n", descriptionSuitableForTestResult(webFrame).utf8().data());

    return true;
}

bool FrameLoadDelegate::didFirstVisuallyNonEmptyLayoutInFrame(WebView*, WebFrame*)
{
    return true;
}

bool FrameLoadDelegate::didDisplayInsecureContent(WebView* webView)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("didDisplayInsecureContent\n");
    return true;
}

bool FrameLoadDelegate::didRunInsecureContent(WebView* webView)
{
    if (!done && gTestRunner->dumpFrameLoadCallbacks())
        TESTRUNNER_OUTPUT("didRunInsecureContent\n");
        
    return true;
}

/*
HRESULT FrameLoadDelegate::onNotify(_In_opt_ IWebNotification* notification)
{
    _bstr_t notificationName;
    HRESULT hr = notification->name(&notificationName.GetBSTR());
    if (FAILED(hr))
        return hr;

    static _bstr_t webViewProgressFinishedNotificationName(WebViewProgressFinishedNotification);

    if (!wcscmp(notificationName, webViewProgressFinishedNotificationName))
        webViewProgressFinishedNotification();

    return S_OK;
}
*/

void FrameLoadDelegate::webViewProgressFinishedNotification()
{
    if (!done && gTestRunner->dumpProgressFinishedCallback())
        TESTRUNNER_OUTPUT("postProgressFinishedNotification\n");
}
} // namespace Telescope
#endif // #if TELESCOPE_USING( TELESCOPE_TESTRUNNER )