/*
 * Copyright (C) 2005-2007, 2015 Apple Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if TELESCOPE_USING( TELESCOPE_TESTRUNNER )

#ifndef FrameLoadDelegate_h
#define FrameLoadDelegate_h
#include "WebFrameLoadDelegate.h"

namespace Telescope
{
class AccessibilityController;
class TextInputController;
class GCController;
class WebView;

class FrameLoadDelegate : public WebFrameLoadDelegate {
    WTF_MAKE_FAST_ALLOCATED;
public:
    FrameLoadDelegate();
    virtual ~FrameLoadDelegate();

    void processWork();

    void resetToConsistentState();
    /*
    AccessibilityController* accessibilityController() const { return m_accessibilityController.get(); }
    */
    virtual bool didStartProvisionalLoadForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didReceiveServerRedirectForProvisionalLoadForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didFailProvisionalLoad(WebView* webView, WebFrame* webFrame);
    virtual bool didCommitLoadForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didReceiveTitle(WebView* webView, const WTF::String&, WebFrame* webFrame);
    virtual bool didFinishLoadForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didFailLoad(WebView* webView, WebFrame* webFrame);
    virtual bool didChangeLocationWithinPageForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool willPerformClientRedirectToURL(WebView*, const WTF::String& url, double delaySeconds, double fireDate, WebFrame*);
    virtual bool didCancelClientRedirectForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool windowScriptObjectAvailable(WebView* webView, JSContextRef, JSObjectRef windowObject);

    virtual bool didFinishDocumentLoadForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didHandleOnloadEventsForFrame(WebView* webView, WebFrame* webFrame);
    virtual bool didFirstVisuallyNonEmptyLayoutInFrame(WebView* webView, WebFrame* webFrame);

    virtual bool didDisplayInsecureContent(WebView* webView);
    virtual bool didRunInsecureContent(WebView* webView);
    virtual bool didClearWindowObjectForFrameInScriptWorld(WebView*, WebFrame*);

private:
    void didClearWindowObjectForFrameInStandardWorld(WebFrame*);

    void locationChangeDone(WebFrame*);
    void webViewProgressFinishedNotification();

    std::unique_ptr<GCController> m_gcController;
    /*
    std::unique_ptr<AccessibilityController> m_accessibilityController;
    std::unique_ptr<TextInputController> m_textInputController;
    */
};

} // namespace Telescope
#endif // FrameLoadDelegate_h

#endif // #if TELESCOPE_USING( TELESCOPE_TESTRUNNER )